<?xml version="1.0"?>
<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!--  American Printing House for the Blind, Inc.
      Copyrifht (c) 2004, All Rights Reserved.

     Convert Daisy3/NISO Dtbook 1.1.0 xml to xhtml,
     using the mxxml 4 DOM to transform the document to an
  	 xhtml document text. Most dtbook specific elements
  	 are converted to div with class attribute by element
  	 name.

  	 To use the output in a browser, several post transform
  	 steps are needed. Add the amp entity to any ampesand
  	 characters. Replace the xhtml root element with the formal
  	 '<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" lang="en">' + xhtmlNodes + '</html>'
  	 and format the document with line breaks before
  	 saving into a text file.

	The html root element was not used to  avoids the undesirable
	transform output of html formatted text that is not XML DOM
	compliant and it also avoids xmlns attributes that
	appear when the xhtml namwspace is used.

	2004-11-1: fix typo in level5 to select h5.
			   in li added line to select.
-->
<xsl:output method="html" omit-xml-declaration="yes" />

<!-- dtbook - top level element - MUST NOT have xml namespace for xhtml -->
<xsl:template match="dtbook">
<html xmlns="http://www.w3.org/1999/xhtml">
<xsl:choose>
<xsl:when test="@xml:lang != ''">
<xsl:attribute name="lang"><xsl:value-of select="@xml:lang"/>
</xsl:attribute>
</xsl:when>
<xsl:otherwise>
<xsl:attribute name="lang">en-US</xsl:attribute>
</xsl:otherwise>
</xsl:choose>
<xsl:apply-templates select="head | book" />
</html>
</xsl:template>

<!-- head - process head section elements -->
<xsl:template match="head">
<head>
<xsl:apply-templates select="title | link | meta | style" />
</head>
</xsl:template>

<!-- head section elements -->
<!-- title - PCDATA - i18n -->
<xsl:template match="title">
<xsl:copy><xsl:apply-templates select="@xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- link - EMPTY - attrs | charset | href | hreflang | type | rel | rev media -->
<xsl:template match="linkk">
<xsl:copy><xsl:apply-templates select="@id | @class | @style | @title | @xml:lang | @dir | @smilref | @charset | @href | @hreflang | @type | @rel | @rev | @media" /></xsl:copy>
</xsl:template>

<!-- meta - EMPTY - i18n | http-equiv | name | content | scheme -->
<xsl:template match="meta">
<xsl:element name="meta"><xsl:apply-templates select="@xml:lang | @dir | @smilref | @http-equiv | @name | @content | @scheme" /></xsl:element>
</xsl:template>

<!-- style - PCDATA -  title | i19n | type | media -->
<xsl:template match="style">
<xsl:copy><xsl:apply-templates select="@title | @xml:lang | @dir | @smilref | @type | @media | text()" /></xsl:copy>
</xsl:template>

<!-- book - process book section elements -->
<xsl:template match="book">
<body>
<xsl:apply-templates select="frontmatter | bodymatter | rearmatter" />
</body>
</xsl:template>

<!-- frontmatter - doctitle | docauthor | level | level1 | block - attrs -->
<xsl:template match="frontmatter">
<xsl:comment> Start of front matter </xsl:comment>
<div><xsl:attribute name="class">frontmatter</xsl:attribute><xsl:apply-templates select="doctitle | docauthor | level | level1 | p | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | h1 | h2 | h3 | h4 | h5 | h6 | blockquote | notice | prodnote | sidebar | note | imggroup | annotation | @id | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
<xsl:comment> End of front matter </xsl:comment>
</xsl:template>

<!-- bodymatter - level | level1 | block - attrs -->
<xsl:template match="bodymatter">
<xsl:comment> Start of body matter </xsl:comment>
<div><xsl:attribute name="class">bodymatter</xsl:attribute><xsl:apply-templates select="level | level1 | p | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | h1 | h2 | h3 | h4 | h5 | h6 | blockquote | notice | prodnote | sidebar | note | imggroup | annotation | @id | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
<xsl:comment> End of body matter </xsl:comment>
</xsl:template>

<!-- rearmatter - level | level1 | block - attrs -->
<xsl:template match="rearmatter">
<xsl:comment> Start of rear matter </xsl:comment>
<div><xsl:attribute name="class">rearmatter</xsl:attribute><xsl:apply-templates select="level | level1 | p | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | h1 | h2 | h3 | h4 | h5 | h6 | blockquote | notice | prodnote | sidebar | note | imggroup | annotation | @id | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
<xsl:comment> End of rear matter </xsl:comment>
</xsl:template>

<!-- level - levelhd | block | inlineinblock | level | doctitle | docauthor - attrs| depth -->
<xsl:template match="level">
<div><xsl:attribute name="class">level</xsl:attribute><xsl:apply-templates select="levelhd | level | doctitle | docauthor | p | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | a | cite | caption | samp | kbd | pagenump | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | @id | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
</xsl:template>

<!--  level1 - h1 | level2 | block | inlineinblock | doctitle | docauthor - attrs -->
<xsl:template match="level1">
<div><xsl:attribute name="class">level1</xsl:attribute><xsl:apply-templates select="h1 | level2 | p | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | a | cite | caption | samp | kbd | pagenump | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | @id | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
</xsl:template>

<!-- level2 - h2 | level3 | block | inlineinblock - attrs -->
<xsl:template match="level2">
<div><xsl:attribute name="class">level2</xsl:attribute><xsl:apply-templates select="h2 | level3 | p | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | a | cite | caption | samp | kbd | pagenump | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | @id | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
</xsl:template>

<!-- level3 - h3 | level4 | block | inlineinblock - attrs -->
<xsl:template match="level3">
<div><xsl:attribute name="class">level3</xsl:attribute><xsl:apply-templates select="h3 | level4 | p | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | a | cite | caption | samp | kbd | pagenump | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | @id | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
</xsl:template>

<!-- level4 - h4 | level5 | block | inlineinblock - attrs -->
<xsl:template match="level4">
<div><xsl:attribute name="class">level4</xsl:attribute><xsl:apply-templates select="h4 | level5 | p | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | a | cite | caption | samp | kbd | pagenump | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | @id | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
</xsl:template>

<!-- level5 - h5 | level6 | block | inlineinblock - attrs -->
<xsl:template match="level5">
<div><xsl:attribute name="class">level5</xsl:attribute><xsl:apply-templates select="h5 | level6 | p | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | a | cite | caption | samp | kbd | pagenump | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | @id | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
</xsl:template>

<!-- level6 - h6 | block | inlineinblock - attrs -->
<xsl:template match="level6">
<div><xsl:attribute name="class">level6</xsl:attribute><xsl:apply-templates select="h6 | p | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | a | cite | caption | samp | kbd | pagenump | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | @id | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
</xsl:template>

<!-- dtbookblock unique elements
     author | notice | prodnote | sidebar | note | imggroup | annotation
-->

<!-- levelhd - inline - attrs
	 convert to div -->
<xsl:template match="levelhd">
<div><xsl:attribute name="class">levelhd</xsl:attribute><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
</xsl:template>

<!-- doctitle - inline - attrs
     convert to a div -->
<xsl:template match="doctitle">
<div><xsl:attribute name="class">doctitle</xsl:attribute><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
</xsl:template>

<!-- docauthor - inline - attrs
     convert to a div -->
<xsl:template match="docauthor">
<div><xsl:attribute name="class">docaauthor</xsl:attribute><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
</xsl:template>

<!-- author -inline - attrs
     convert to a div with class docauthor -->
<xsl:template match="author">
<div><xsl:attribute name="class">author</xsl:attribute><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
</xsl:template>

<!-- notice - inline - attrs
     convert to a div with class notice -->
<xsl:template match="notice">
<div><xsl:attribute name="class">notice</xsl:attribute><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
</xsl:template>

<!-- prodnote - inline - attrs
     remove from text as dtbook specific text -->
<xsl:template match="prodnote"></xsl:template>

<!-- sidebar - flow | hd - attrs
     convert to a div with class sidebar -->
<xsl:template match="sidebar">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | annoref | noteref | p | list | dl | div | blockquote | hr | table | address | author | notice | prodnote | sidebar | note | annotation | hd | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- hd - inline - attrs
     convert to a div with class hd -->
<xsl:template match="hd">
<div><xsl:attribute name="class">hd</xsl:attribute><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
</xsl:template>

<!-- note - block | inlineinblock - attrsrqd -->
<xsl:template match="note">
<div><xsl:attribute name="class">note</xsl:attribute><xsl:apply-templates select="p | lIST | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | a | cite | caption | samp | kbd | pagenump | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | pagenum | @id | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
</xsl:template>

<!-- annotation - block | inlineinblock - attrsrqd
     convert to div with class of annotation -->
<xsl:template match="annotation">
<div><xsl:attribute name="class">annotation</xsl:attribute><xsl:apply-templates select="p | list| dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | a | cite | caption | samp | kbd | pagenump | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | pagenum | @id | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
</xsl:template>

<!-- line - inline - attrs
     convert to div element -->
<xsl:template match="line">
<div><xsl:attribute name="class">line</xsl:attribute><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></div>
</xsl:template>

<!-- inline special dtbook
-->
<!-- imggroup - prodnote | img | caption - attrs
     convert to div with class imggroup -->
<xsl:template match="imggroup">
<span><xsl:attribute name="class">imggroup</xsl:attribute><xsl:apply-templates select="prodnote | img | caption  | kbd | pagenum | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></span>
</xsl:template>

<!-- dtbook inline -
		sent | w | pagenum | prodnote | annoref | noteref
-->
<!-- sent -inlines (#PCDATA | phrase | special | pagenum |
           w | prodnote | annoref | noteref - attrs
           convert to span with class sent -->
<xsl:template match="sent">
<span><xsl:attribute name="class">sent</xsl:attribute><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | pagenum | w | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></span>
</xsl:template>

<!-- w - inlinew (#PCDATA | phrase | special) - attrs
	 convert to span with class w -->
<xsl:template match="w">
<span><xsl:attribute name="class">w</xsl:attribute><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | @id | @class | @style | @title | @xml:lang | @dir | @smilref | page | text()" /></span>
</xsl:template>

<!-- pagenum - PCDATA -  attrsrqd | page (front | normal | special) 'normal'
	 convert to span with class pagenum -->
<xsl:template match="pagenum">
<span><xsl:attribute name="class">pagenum</xsl:attribute><xsl:apply-templates select="@id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></span>
</xsl:template>

<!-- annoref - PCDATA - attrs | idref | type
	 convert to span woth class annoref -->
<xsl:template match="annoref">
<span><xsl:attribute name="class">annoref</xsl:attribute><xsl:apply-templates select="@id | @class | @style | @title | @xml:lang | @dir | @smilref |  @idref | @type | text()" /></span>
</xsl:template>

<!-- noteref - PCDATA - attrs | idref | type
	 convert to notefef with class notefef -->
<xsl:template match="noteref">
<span><xsl:attribute name="class">noteref</xsl:attribute><xsl:apply-templates select="@id | @class | @style | @title | @xml:lang | @dir | @smilref |  @idref | @type | text()" /></span>
</xsl:template>


<!-- - block elements
	block: "p | list | dl | div | blockquote | hr |
			img | table | address"
	flow: phrase | special |
			"#PCDATA | em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum"
		and blocknoimggroup:
		   "p | list | dl | div | blockquote | hr | table | address | line 
		with dtbookblocknoimggroup:
		    "author | notice | prodnote | sidebar | note | annotation
	inlineinblock:
	 	   "a | cite | caption | samp | kbd"
-->
<!-- p - inline | list | dl -  attrs -->
<xsl:template match="p">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | list | dl | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- list - li - attrs +
     convert to ul since dtb has explicit text -->
<xsl:template match="list">
<ul><xsl:apply-templates select="@id | @class | @style | @title | @xml:lang | @dir" /><xsl:apply-templates select="li" /></ul>
</xsl:template>

<!-- li - flow | lic - attrs -->
<xsl:template match="li">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | annoref | noteref | p | line | list | dl | div | blockquote | hr | table | address | author | notice | prodnote | sidebar | note | annotation | lic | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>
<!-- lic -inline - attrs -->
<xsl:template match="lic">
<span><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></span>
</xsl:template>

<!-- dl - dt | dd - attrs -->
<xsl:template match="dl">
<xsl:copy><xsl:apply-templates select="dt | dd | @id | @class | @style | @title | @xml:lang | @dir" /></xsl:copy>
</xsl:template>
<!-- dt - inline - attrs -->
<xsl:template match="dt">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>
<!-- dd - flow - attrs -->
<xsl:template match="dd">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | annoref | noteref | p | list | dl | div | blockquote | hr | table | address | author | notice | prodnote | sidebar | note | annotation | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- div - block | inlineinblock - attrs and level -->
<xsl:template match="div">
<xsl:copy><xsl:apply-templates select="docauthor | doctitle | p | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | a | cite | caption | samp | kbd | pagenump | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | pagenum | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- blockquote - block - attrs and cite -->
<xsl:template match="blockquote">
<xsl:copy><xsl:apply-templates select="author | p | list | dl | div | blockquote | hr | img | imggroup | table | address | line | author | notice | prodnote | sidebar | note | annotation | @id | @class | @style | @title | @xml:lang | @dir" /></xsl:copy>
</xsl:template>

<!-- hr - EMPTY - coreattrs -->
<xsl:template match="hr">
<xsl:copy><xsl:apply-templates select="@id | @class | @style | @title" /></xsl:copy>
</xsl:template>

<!-- address - inline | line - attrs -->
<xsl:template match="address">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | line | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- table:
	Scope: "row | col | rowgroup | colgroup"
	TFrame: "void | above | below | hsides | lhs | rhs | vsides | box | border"
	TRules: "none | groups | rows | cols | all"
	cellhalign: "align (left|center|right|justify|char) | char | charoff
	cellvalign: "valign     (top|middle|bottom|baseline) #IMPLIED"
-->

<!-- table - caption | col | colgroup | thead | tfoot | tbody | tr - 
		    attrs | summary | width | border | frame | rules | cellspacing | cellpadding -->
<xsl:template match="table">
<xsl:copy><xsl:apply-templates select="caption | col | colgroup | thead | tfoot | tbody | tr | @id | @class | @style | @title | @summary | @width | @border | @frame | @rules | @cellspacing | @cellpadding" /></xsl:copy>
</xsl:template>

<!-- caption - inline - attrs (not imgref) -->
<xsl:template match="caption">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- thead - tr - attrs | cellhalign | cellvalign -->
<xsl:template match="thead">
<xsl:copy><xsl:apply-templates select="tr | @id | @class | @style | @title | @xml:lang | @dir | @smilref | @align | @char | @charoff | @valign | text()" /></xsl:copy>
</xsl:template>

<!-- tfoot - tr - attrs | cellhalign | cellvalign -->
<xsl:template match="tfoot">
<xsl:copy><xsl:apply-templates select="tr | @id | @class | @style | @title | @xml:lang | @dir | @smilref | @align | @char | @charoff | @valign | text()" /></xsl:copy>
</xsl:template>

<!-- tbody - tr - attrs | cellhalign | cellvalign -->
<xsl:template match="tbody">
<xsl:copy><xsl:apply-templates select="tr | @id | @class | @style | @title | @xml:lang | @dir | @smilref | @align | @char | @charoff | @valign | text()" /></xsl:copy>
</xsl:template>

<!-- colgroup - col - attrs | cellhalign | cellvalign | span | width -->
<xsl:template match="colgroup">
<xsl:copy><xsl:apply-templates select="col | @id | @class | @style | @title | @xml:lang | @dir | @smilref | @align | @char | @charoff | @valign | @span | @width | text()" /></xsl:copy>
</xsl:template>

<!-- col - EMPTY - attrs | cellhalign | cellvalign | span | width -->
<xsl:template match="col">
<xsl:copy><xsl:apply-templates select="col | @id | @class | @style | @title | @xml:lang | @dir | @smilref | @align | @char | @charoff | @valign | @span | @width | text()" /></xsl:copy>
</xsl:template>

<!-- tr - th | td - attrs | cellhalign | cellvalign -->
<xsl:template match="tr">
<xsl:copy><xsl:apply-templates select="th | td | @id | @class | @style | @title | @xml:lang | @dir | @smilref | @align | @char | @charoff | @valign | text()" /></xsl:copy>
</xsl:template>

<!-- th - flownopagenum (see td)
		- attrs | cellhalign | cellvalign |
		  abbr | axis | headers | scope | rowspan | colspan  -->
<xsl:template match="th">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | annoref | noteref | p | list | dl | div | blockquote | hr | table | address | author | notice | prodnote | sidebar | note | annotation | @id | @class | @style | @title | @xml:lang | @dir | @smilref | @align | @char | @charoff | @valign | @abbr | @axis | @header| @scope | @rowspan | @colspan | text()" /></xsl:copy>
</xsl:template>

<!-- td - flownopagenum:
			inlinenopagenum - phrase | special | sent | w | annoref | noteref
			blocknoimggroup - see above
		- attrs | cellhalign | cellvalign |
		  abbr | axis | headers | scope | rowspan | colspan  -->
<xsl:template match="td">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | annoref | noteref | p | list | dl | div | blockquote | hr | table | address | author | notice | prodnote | sidebar | note | annotation | @id | @class | @style | @title | @xml:lang | @dir | @smilref | @align | @char | @charoff | @valign | @abbr | @axis | @header| @scope | @rowspan | @colspan | text()" /></xsl:copy>
</xsl:template>

<!-- headeings -->
<!-- h1 - inline - attrs -->
<xsl:template match="h1">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- h2 - inline - attrs -->
<xsl:template match="h2">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- h3 - inline - attrs -->
<xsl:template match="h3">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- h4 - inline - attrs -->
<xsl:template match="h4">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- h5 - inline - attrs -->
<xsl:template match="h5">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- h6 - inline - attrs -->
<xsl:template match="h6">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>


<!-- - inline elments
	inline: #PCDATA +
    phrase - em | strong | dfn | code | samp | kbd | cite | abbr | acronym
    special - a | img | imggroup | br | q | sub | sup | span | bdo | linenum
    dtbookinline - sent | w | pagenum | prodnote | annoref | notere
-->

<!-- em - inline - attrs -->
<xsl:template match="em">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- strong - inline - attrs -->
<xsl:template match="strong">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- dfn - inline - attrs -->
<xsl:template match="dfn">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- code - inline - attrs and add xml:space default -->
<xsl:template match="code">
<xsl:copy><xsl:attribute name="xml:space">default</xsl:attribute><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- samp - inline - attrs and add xml:space default -->
<xsl:template match="samp">
<xsl:copy><xsl:attribute name="xml:space">default</xsl:attribute><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- kbd - inline - attrs -->
<xsl:template match="kbd">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- cite - inline - attrs -->
<xsl:template match="cite">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- abbr - inline - attrs -->
<xsl:template match="abbr">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- acronym - inline - attrs and add pronounce -->
<xsl:template match="acronym">
<xsl:copy><xsl:attribute name="pronounce">no</xsl:attribute><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | @pronounce | text()" /></xsl:copy>
</xsl:template>

<!-- a - inlinenoa - attrs | src | alt | height | width | longdesc -->
<xsl:template match="a">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | @type | @href | @hreflang | @rel | @rev | @accesskey | @tabindex | text()" /></xsl:copy>
</xsl:template>

<!-- img - EMPTY - attrs | src | alt | height | width | longdesc -->
<xsl:template match="img">
<xsl:copy><xsl:apply-templates select="@id | @class | @style | @title | @xml:lang | @dir | @smilref | @src | @alt | @height | @width | @longdesc" /></xsl:copy>
</xsl:template>

<!-- br | EMPTY - coreattrs -->
<xsl:template match="br">
<xsl:copy><xsl:apply-templates select="@id | @class | @style | @title" /></xsl:copy>
</xsl:template>

<!-- q - inline - attrs and cite (for a URI) -->
<xsl:template match="q">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | @cite | text()" /></xsl:copy>
</xsl:template>

<!-- sub - inline - attrs -->
<xsl:template match="sub">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- sup - inline - attrs -->
<xsl:template match="sup">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- span - inline - attrs -->
<xsl:template match="span">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- bdo - inline - coreattrs and lang, dir - exclude xml:lang -->
<xsl:template match="bdo">
<xsl:copy><xsl:apply-templates select="em | strong | dfn | code | samp | kbd | cite | abbr | acronym | a | img | imggroup | br | q | sub | sup | span | bdo | linenum | sent | w | pagenum | prodnote | annoref | noteref | @id | @class | @style | @title | @xml:lang | @dir | @smilref | text()" /></xsl:copy>
</xsl:template>

<!-- linenum - #PCDATA - attrs -->
<xsl:template match="linenum">
<xsl:copy><xsl:apply-templates select="@id | @class | @style | @title | @smilref | text()" /></xsl:copy>
</xsl:template>


<!-- - attributes - -->
<!-- core -->
<xsl:template match="@id">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@class">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@style">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@title">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@xml:lang">
<xsl:attribute name="lang"><xsl:value-of select="@*" /></xsl:attribute>
</xsl:template>

<xsl:template match="@dir">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<!-- a -->
<xsl:template match="@href">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@accesskey">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@tabindex">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<!-- img -->
<xsl:template match="@src">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@alt">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@longdesc">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@height">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@width">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<!-- meta, link and style -->
<xsl:template match="@charset">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@hreflang">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@media">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@rel">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@rev">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@type">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@http-equiv">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@name">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@content">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@scheme">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<!-- ol - number at start -->
<xsl:template match="@start">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<!-- table attributes -->
<xsl:template match="@summary">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@border">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@frame">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@rules">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@cellspacing">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@cellpadding">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@align">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@char">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@charoff">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@valign">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<!-- colgroup -->
<xsl:template match="@colgroup">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@width">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<!-- th and td -->
<xsl:template match="@abbr">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@axis">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@headers">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@scope">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@rowspan">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@colspan">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<!-- dtbook attribues -->
<xsl:template match="@page">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@idref">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

<xsl:template match="@smilref">
<xsl:copy><xsl:value-of select="@*" /></xsl:copy>
</xsl:template>

</xsl:stylesheet>
